from datetime import datetime
import enum
from typing import List, Annotated, Optional

from fastapi_users.db import SQLAlchemyBaseUserTable
from sqlalchemy import ForeignKey, text
from sqlalchemy.orm import Mapped, mapped_column, relationship, DeclarativeBase


intpk = Annotated[int, mapped_column(primary_key=True)]
str_25 = Annotated[str, 25]
created_at = Annotated[
    datetime,
    mapped_column(server_default=text("TIMEZONE('utc', now())")),
]
updated_at = Annotated[
    datetime,
    mapped_column(
        server_default=text("TIMEZONE('utc', now())"),
        onupdate=datetime.utcnow,
    ),
]


class Base(DeclarativeBase):
    pass


class UserRoles(enum.Enum):
    """
    Доступные роли юзеров/клиентов
    """

    admin = "Администратор"
    director = "Руководитель"
    worker = "Работник"
    client = "Клиент"


class UserRoleAssociation(Base):
    """
    Модель таблицы связи Many2Many для Role & User
    """

    __tablename__ = "user_role_association"
    user_id: Mapped[int] = mapped_column(
        ForeignKey("user.id"),
        primary_key=True,
    )
    role_id: Mapped[int] = mapped_column(
        ForeignKey("role.id"),
        primary_key=True,
    )
    role: Mapped["Role"] = relationship()


class Role(Base):
    """
    Модель таблицы role
    """

    __tablename__ = "role"

    id: Mapped[intpk]
    role: Mapped[Optional[UserRoles]]


class User(SQLAlchemyBaseUserTable[int], Base):
    """
    Модель таблицы user
    """

    __tablename__ = "user"
    id: Mapped[intpk]
    name: Mapped[str_25]
    patronim: Mapped[str_25]
    last_name: Mapped[str_25]
    roles: Mapped[List["Role"]] = relationship(
        secondary="user_role_association",
    )
    equipment: Mapped[List["Equipment"]] = relationship()
    eq_locations: Mapped[List["UserLocationAssociation"]] = relationship()


class UserLocationAssociation(Base):
    """таблица связи Many2Many между user & location"""

    __tablename__ = "user_location_association"
    client_id: Mapped[int] = mapped_column(
        ForeignKey("user.id"),
        primary_key=True,
    )
    location_id: Mapped[int] = mapped_column(
        ForeignKey("location.id"), primary_key=True
    )
    location: Mapped["Location"] = relationship()


class Location(Base):
    """
    Адреса клиентов где установленно оборудование
    """

    __tablename__ = "location"
    id: Mapped[intpk]
    address: Mapped[str]
    client: Mapped[int]


class Equipment(Base):
    """ """

    __tablename__ = "equipment"
    id: Mapped[intpk]
    name: Mapped[str]
    eq_type: Mapped[int]
    put_into_operation_at: Mapped[datetime]
    owner_id: Mapped[int] = mapped_column(ForeignKey("user.id"))
    #  Определение связи many2one
    location_id: Mapped[int] = mapped_column(ForeignKey("location.id"))
    location: Mapped["Location"] = relationship()
